function [TauTilde,KTilde] = TwoPhaseAlg(varargin)
% Runs the 2-phase algorithm
% Example: TwoPhaseAlg(meas,noiseLevel,K,dynTol,MaxIters,mMax)
%
% INPUTS:
% meas: Fourier coeffs on [-fC:fC]
% noiseLevel: noise level per coefficient in the Fourier domain (see the
% accompanying paper)
% K: number of impulses (set K = Inf to estimate the number of impulses)
% dynTol: max dynamic range allowed
% MaxIters: max iterations of Newton's algorithm
% mMax: max line search iterations in Newton's algothim
%
% With no inputs, TwoPhaseAlg runs the demo (Figure 1 of the accompanying
% paper "Greed is Super: A Fast Algorithm for Super-Resolution" by 
% A. Ektekhari and M. B. Wakin)
%
% OUTPUTS:
% TauTilde: Estimated impulse positions
% KTilde: Estimated number of impulses
%
% Copyright 2015, Armin Eftekhari and Michael Wakin
%
% This file is part of the Greedy Super-Resolution Toolbox version 1.0.
%
%    The Greedy Super-Resolution Toolbox is free software: you can 
%    redistribute it and/or modify it under the terms of the GNU General 
%    Public License as published by the Free Software Foundation, either 
%    version 3 of the License, or (at your option) any later version.
%
%    The Greedy Super-Resolution Toolbox is distributed in the hope that it 
%    will be useful, but WITHOUT ANY WARRANTY; without even the implied 
%    warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
%    See the GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with the Greedy Super-Resolution Toolbox.  If not, 
%    see <http://www.gnu.org/licenses/>.
%

if length(varargin) <= 5
    mMax = 10; % Max line search iterations in Newton's algorithm
    if length(varargin) <= 4
        MaxIters = 50; % Max iterations in Newton's algorithm
        if length(varargin) <= 3
            dynTol = 20; % Max dynamic range allowed
            if length(varargin) <= 2
                K = Inf; % Set K = Inf to estimate the number of impulses
                if length(varargin) == 1
                    display('Noise lever required');
                    return
                elseif isempty(varargin) % Run the demo
                    clc; close all;
                    fC = 50; % Cut-off frequency
                    noiseLevel = 0; % Noise free
                    Tau = [0.2995  0.3663  0.4332  0.5000  0.5668  0.6337  0.7005]'; % Impulse positions
                    Alpha = [10  -1  1  -3  2  -5  2]'; % Amplitudes
                    Freq = -fC:fC;
                    
                    % Data
                    F = exp(-1i*2*pi*Freq'*Tau'); % Fourier matrix
                    meas = F*Alpha; % Frequency domain measurements
                end
            end
        end
    end
end

if length(varargin) >= 2
    meas = varargin{1};
    noiseLevel = varargin{2};
    fC = (length(meas)-1)/2;
    if length(varargin) >= 3
        K = varargin{3};
        if length(varargin) >= 4
            dynTol = varargin{4};
            if length(varargin) >= 5
                MaxIters = varargin{5};
                if length(varargin) >= 6
                    mMax = varargin{6};
                end
            end
        end
    end
end




% DPSWF widths
c1 = 3/2;
sigma1 = c1/(2*fC+1); % Resolution
c2 = 3/2*c1;
sigma2 = c2/(2*fC+1); % Resolution

e = @(tau,f) exp(-1i*2*pi*[-f:f]'*tau');

% Time grid
timeRes = 1/(1e3*fC);
T = (0:timeRes:1)';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Algorithm  I (initialization)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Filter the frequency measurments with the top DPSS
psiHat = dpss(2*fC+1,(2*fC+1)*sigma1,1);
Meas = psiHat.*meas;
X = real((e(T,fC))'*Meas);

if isempty(varargin) % Demo
    
    % Plot the low-frequency input in time
    x = real((e(T,fC))'*meas);
    figure; plot(T,x,'LineWidth',3); hold on;
    stem(Tau,x(floor(Tau/timeRes)),'MarkerSize',6,'MarkerFaceColor','r');
    set(gca,'XTick',0:.2:1,'FontSize',20);
    axis([0 1 -700 1200]);
    title('Low-frequency input in time');
    xlabel('Time')
    %     print(gcf,[pwd,'\Super-res-demo'],'-dpdf');
    
    % Plot the top DPSWF
    psi = real((e(T,fC))'*psiHat);
    psi = fftshift(psi);
    figure; plot(T-1/2,psi,'LineWidth',3)
    set(gca,'XTick',-1/2:.2:1/2,'FontSize',20);
    axis([-1/2 1/2 -1 10]);
    title('Top DPSWF in time');
    xlabel('Time')
    %     print(gcf,[pwd,'\DPSWF-example'],'-dpdf');
    
    % Plot the filtered low-frequency input in time
    figure; plot(T,X,'LineWidth',3); hold on;
    stem(Tau,X(floor(Tau/timeRes)),'MarkerSize',6,'MarkerFaceColor','r');
    set(gca,'XTick',0:.2:1,'FontSize',20);
    title('Low-frequency input filtered with the top DPSWF');
    xlabel('Time'); hold off;
    %     print(gcf,[pwd,'\Super-res-demo-filtered'],'-dpdf');
    
end


% Main loop
counter = 1:length(T);
Neigh = 2*round(sigma1*length(T));
counter = [counter(end-Neigh+1:end) counter counter(1:Neigh)];
Rho = []; % Stores the impulse locations
peak = []; % Keeps track of dynamic range
StillSignal = 1; % Continue removing impulses until noise level is reached
k = 1; % Tracks the number of detected impulses

while StillSignal
    
    % Find the peak of filtered signal
    [~,rho] = max(abs(X));
    peak = [peak X(rho)]; % Store a rough estimate of amplitudes
    Rho = [Rho;rho]; % Store the estimated impulse position
    
    % Erase the neighborhood of rho of radius sigma1
    X(counter(rho:rho+2*Neigh)) = 0;

    % Have we reached the noise level? 
    if k >= K % If number of impulses is known
        StillSignal = 0;
    elseif  (max(abs(X)) < max([2*square((2*fC+1)*log(2*fC+1))*noiseLevel,peak(1)/dynTol])) && (K == Inf) % If number of impulses is unknown
        StillSignal = 0;
    else
        k = k+1;
    end
    
end

Rho = sort(Rho,'ascend');
Rho = Rho*timeRes; % Initialization for Algorithm II
KTilde = k; % Estimated number of impulses
PhaseITau = Rho;


if isempty(varargin)
    error1 = H_dist(PhaseITau,Tau); % Compute the Hausdorff distance between the estimated positions PhaseITau and the true positions Tau
    display(['Error of Phase I = ',num2str(error1,15)]);
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Algorithm II (local optimization)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Filter the measurment signal with the top DPSS with width sigma2
psiHat = dpss(2*fC+1,(2*fC+1)*sigma2,1);
Meas = psiHat.*meas;

% Setup for Newton's method
epsilon = sigma1/2; % Threshold for active coordinates
eta = 1e-16; % Termination threshold

% Projected Newton's steps
for iter = 1:MaxIters
    [Rho,epsilon] = ProjectedNewtonStep(Rho,epsilon,fC,sigma1,Meas,psiHat,PhaseITau,eta,mMax);
end
TauTilde = Rho;

if isempty(varargin)
    error2 = H_dist(TauTilde,Tau); % Compute the Hausdorff distance between the estimated positions TauTilde and the true positions Tau
    display(['Error of Phase II = ',num2str(error2,15)]);
end


end